@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,7 +48,7 @@
 #define SNIPPET_DETAILS_FONT_SIZE 10
 
 using namespace mforms;
-using namespace MySQL::Geometry;
+using namespace base;
 
 //--------------------------------------------------------------------------------------------------
 // TODO: move this to a central cairo helper class.
@@ -85,7 +85,7 @@ private:
   double _title_offset;               // Vertical position of the title.
   double _description_offset;         // Ditto for description.
   double _title_width;                // Width of the (possibly shortened) title. For text decoration.
-  MySQL::Geometry::Rect _bounds;      // The link's bounds when it was drawn the last time.
+  base::Rect _bounds;      // The link's bounds when it was drawn the last time.
   int _text_height;
   bool _enabled;                      // Draw the button in enabled or disabled state.
 
@@ -107,7 +107,7 @@ public:
 
   //------------------------------------------------------------------------------------------------
 
-  ~Snippet()
+  virtual ~Snippet()
   {
     if (_icon != NULL)
       cairo_surface_destroy(_icon);
@@ -145,7 +145,7 @@ public:
     _text_height = (int) ceil(title_extents.height + description_extents.height + SNIPPET_TEXT_SPACING);
   }
 
-  virtual void paint(cairo_t* cr, MySQL::Geometry::Rect bounds, bool hot, bool active)
+  virtual void paint(cairo_t* cr, base::Rect bounds, bool hot, bool active)
   {
     _bounds = bounds;
     cairo_save(cr);
@@ -200,7 +200,7 @@ public:
         else
           cairo_set_source_rgb(cr, 34 / 255.0, 34 / 255.0, 34 / 255.0);
     double offset= bounds.left() + SNIPPET_PADDING + 
-      (icon_width > 0) ? icon_width + SNIPPET_ICON_SPACING : 0;
+      ((icon_width > 0) ? icon_width + SNIPPET_ICON_SPACING : 0);
 
     cairo_move_to(cr, offset, bounds.top()  + _title_offset + text_offset);
     cairo_show_text(cr, _shortened_title.c_str());
@@ -250,7 +250,7 @@ public:
 
   //------------------------------------------------------------------------------------------------
 
-  MySQL::Geometry::Rect bounds()
+  base::Rect bounds()
   {
     return _bounds;
   };
@@ -295,7 +295,7 @@ public:
 /**
   * Internal class containing a list of snippet entries in a scrollable list.
   */
-class SnippetList : public mforms::DrawBox, public base::trackable
+class SnippetList : public mforms::DrawBox
 {
 private:
   DbSqlEditorSnippets* _model;     // The model containing the actual snippet data.
@@ -603,7 +603,7 @@ public:
         Snippet* snippet = snippet_from_point(x, y);
         if (snippet != NULL && snippet == _selected_snippet)
         {
-          MySQL::Geometry::Rect bounds = snippet->bounds();
+          base::Rect bounds = snippet->bounds();
 
           int left = (int) bounds.left();
           int dummy = left;
@@ -722,7 +722,6 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   padding_box->set_back_color("#ffffff");
   _help_box->set_back_color("#ffffff");
   _help_box->add(padding_box, true, true);
-  padding_box->release();
 #else
   _help_text->set_back_color("#ebebeb");
   _help_box->add(_help_text, true, true);
@@ -755,7 +754,6 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   
   Box* snippet_page = manage(new Box(false));
   add_page(snippet_page, "Snippets");
-  snippet_page->release();
 
   // Separate box since we need a border around the snippet list, but not the toolbar.
   Box* snippet_list_border = manage(new Box(false));
@@ -777,11 +775,9 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
   ToolBar* toolbar = prepare_toolbar();
   _snippet_toolbar = toolbar;
   snippet_page->add(toolbar, false, true);
-  toolbar->release();
 
   snippet_list_border->add(_snippet_box, true, true);
   snippet_page->add(snippet_list_border, true, true);
-  snippet_list_border->release();
 
   scoped_connect(_snippet_list->signal_selection_changed(), boost::bind(&QuerySidePalette::snippet_selection_changed, this));
   snippet_selection_changed();
@@ -791,11 +787,6 @@ QuerySidePalette::QuerySidePalette(const SqlEditorForm::Ref &owner)
 
 QuerySidePalette::~QuerySidePalette()
 {
-  _help_box->release();
-  _help_toolbar->release();
-  _help_text->release();
-  _snippet_box->release();
-  _snippet_list->release();
 }
 
 //--------------------------------------------------------------------------------------------------
