@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -144,6 +144,10 @@ HostMachinePage::HostMachinePage(WizardForm* host, wb::WBContext *wb)
 
 void HostMachinePage::host_selection_changed()
 {
+  // Mark a local connection only if the user selected localhost. This way we can allow
+  // an IP address like 127.0.0.1 to be used like a remote one (e.g. for port forwarding).
+  values().gset("isLocal", _localhost_button.get_active() ? 1 : 0);
+
   bool flag= _networkhost_button.get_active();
   _ip_address_block.set_enabled(flag);
 
@@ -262,23 +266,21 @@ bool DatabaseServerPage::skip_page()
 
 void DatabaseServerPage::enter(bool advancing)
 {
+  wizard()->clear_problem();
+
   if (advancing)
   {
-    db_mgmt_ConnectionRef conn(_context->get_grt());
-    conn->name(values().get_string("host_name", "new_connection"));
-    conn->parameterValues().gset("hostName", values().get_string("host_name", "new_connection"));
-    
     if (!_conn_name)
     {
+      db_mgmt_ConnectionRef conn(_context->get_grt());
+      conn->name(values().get_string("host_name", "new_connection"));
+      conn->parameterValues().gset("hostName", values().get_string("host_name", "new_connection"));
+
       _panel.init(_context->get_root()->rdbmsMgmt(), conn);
     
-      _conn_name= _panel.get_name_entry();
+      _conn_name = _panel.get_name_entry();
+      _conn_name->set_value(conn->name());    
     }
-    else
-    {
-      _panel.get_be()->set_connection(conn);
-    }
-    _conn_name->set_value(conn->name());    
   }
 }
 
@@ -286,6 +288,17 @@ void DatabaseServerPage::enter(bool advancing)
 
 bool DatabaseServerPage::advance()
 {
+  // Check if the instance name already exists.
+  grt::ListRef<db_mgmt_Connection> connections = _context->get_root()->rdbmsMgmt()->storedConns();
+  std::string name = base::trim(_conn_name->get_string_value());
+  GRTLIST_FOREACH(db_mgmt_Connection, connections, connection)
+  {
+    if ((*connection)->name() == name)
+    {
+      wizard()->set_problem(strfmt(_("A connection with the name %s exists already. Please choose another one."), name.c_str()));
+      return false;
+    }
+  }
   return true;
 }
 
@@ -298,6 +311,8 @@ void DatabaseServerPage::leave(bool advancing)
     _panel.get_be()->save_changes();
 
     db_mgmt_ConnectionRef connection(_panel.get_connection());
+    std::string name = base::trim(_conn_name->get_string_value());
+    connection->name(name);
     values().set("connection", connection);
 
     std::string machine;
@@ -342,7 +357,7 @@ TestDatabaseSettingsPage::TestDatabaseSettingsPage(WizardForm* host)
            boost::bind(&TestDatabaseSettingsPage::get_server_platform, this),
            _("Querying server OS type..."));
 
-/*XXX TODO
+/*XXX Improve rights checks.
   add_task(_("Check Account Privileges"),
            boost::bind(&TestDatabaseSettingsPage::get_server_platform, this),
            _("Checking if account being used has enough privileges..."));
@@ -659,6 +674,8 @@ void HostAndRemoteTypePage::enter(bool advancing)
 
 void HostAndRemoteTypePage::refresh_profile_list()
 {
+  wizard()->clear_problem();
+
   std::string system = _os_selector.get_string_value();
   
   _type_selector.clear();
@@ -671,6 +688,8 @@ void HostAndRemoteTypePage::refresh_profile_list()
 
 void HostAndRemoteTypePage::toggle_remote_admin()
 {
+  wizard()->clear_problem();
+
   // Clear out any OS value if we don't use remote management.
   if (_no_remote_admin.get_active() && !wizard()->is_local())
   {
@@ -1906,8 +1925,8 @@ db_mgmt_ServerInstanceRef NewServerInstanceWizard::assemble_server_instance()
   
   _instance->serverInfo().gset("sys.script", get_admin_script_for_os(os));
   
-  if (win_management || get_active_page_number() >= 7
-      && values().get_int("customize")) // don't overwrite defaults unless user has had time to enter them
+  if (win_management || (get_active_page_number() >= 7
+      && values().get_int("customize"))) // don't overwrite defaults unless user has had time to enter them
   {
     std::string ini_path= values().get_string("ini_path");
     std::string ini_section= values().get_string("ini_section");
@@ -2045,8 +2064,10 @@ bool NewServerInstanceWizard::is_admin_enabled()
  */
 bool NewServerInstanceWizard::is_local()
 {
-  std::string host= values().get_string("host_name", "localhost");
-  return (host == "localhost") || (host == "127.0.0.1") || (base::trim(host).empty());
+  // A local connection is not defined by an IP address (as port forwarding could be active)
+  // but by the user himself, as he should know if it is a local host or not.
+  int isLocal = values().get_int("isLocal");
+  return isLocal != 0;
 }
 
 //--------------------------------------------------------------------------------------------------
