@@ -90,43 +90,6 @@ static bool close_splash(Gtk::Window *w)
 }
 
 
-static void show_help()
-{
-  printf("mysql-workbench [<options>] [<model file>]\n");
-  printf("Options:\n");
-  printf("  --force-sw-render      Force Xlib rendering\n");
-  printf("  --force-opengl-render  Force OpenGL rendering\n");
-  printf("  --query <connection>   Open a query tab to the named connection\n");
-  printf("  --admin <instance>     Open a administration tab to the named instance\n");
-  printf("  --model <model file>   Open the given EER model file\n");
-  printf("  --script <script file> Execute the given Python or Lua script file\n");
-  printf("  --run <script>         Execute the given code in default language for GRT shell\n");
-  printf("  --run-python <script>  Execute the given code in Python\n");
-  printf("  --run-lua <script>     Execute the given code in Lua\n");
-  printf("  --quit-when-done       Quit Workbench when the script is done\n");
-  printf("  --help, -h             Show command line options and exit\n");
-  printf("  --log-level=<level>    Valid levels are: error, warning, info, debug1, debug2, debug3\n");
-  printf("  --verbose              Enable diagnostics output\n");
-  printf("  --version              Show Workbench version number and exit\n");
-}
-
-static bool parse_loglevel(const std::string& line)
-{
-  bool ret = false;
-
-  const size_t eq_char_pos = line.find("=");
-  if (eq_char_pos != std::string::npos)
-  {
-    std::string level = line.substr(eq_char_pos + 1);
-    level = base::tolower(level);
-    ret = base::Logger::active_level(level);
-    if (ret)
-      printf("Logger set to level '%s'. '%s'\n", level.c_str(), base::Logger::get_state().c_str());
-  }
-
-  return ret;
-}
-
 int main(int argc, char **argv)
 {
 
@@ -159,106 +122,8 @@ int main(int argc, char **argv)
 
   g_set_application_name("MySQL Workbench");
 
-  while (i < argc)
-  {
-    if (strcmp(argv[i], "--force-sw-render") == 0)
-      wboptions.force_sw_rendering= true;
-    else if (strcmp(argv[i], "--force-opengl-render") == 0)
-      wboptions.force_opengl_rendering= true;
-    else if (strcmp(argv[i], "--help") == 0 || strcmp(argv[i], "-h") == 0)
-    {
-      show_help();
-      exit(0);
-    }
-    else if (strcmp(argv[i], "--model") == 0
-            || strcmp(argv[i], "--query") == 0
-            || strcmp(argv[i], "--admin") == 0
-            || strcmp(argv[i], "--script") == 0)
-    {
-      wboptions.open_at_startup_type= argv[i]+2;
-      if (i+1 < argc)
-        wboptions.open_at_startup= argv[++i];
-      else
-      {
-        printf("%s: Missing argument for %s option\n", argv[0], argv[i]);
-        exit(1);
-      }
-    }
-    else if (strcmp(argv[i], "--run") == 0)
-    {
-      if (i+1 < argc)
-        wboptions.run_at_startup= argv[++i];
-      else
-      {
-        printf("%s: Missing argument for --run option\n", argv[0]);
-        exit(1);
-      }
-    }
-    else if (strncmp(argv[i], "--run-", strlen("--run-")) == 0)
-    {
-      wboptions.run_language= argv[i]+strlen("--run-");
-          
-      if (i+1 < argc)
-        wboptions.run_at_startup= argv[++i];
-      else
-      {
-        printf("%s: Missing argument for %s option", argv[0], argv[i-1]);
-        exit(1);
-      }
-    }
-    else if (strcmp(argv[i], "--quit-when-done") == 0)
-      wboptions.quit_when_done= true;
-    else if (strcmp(argv[i], "--version") == 0)
-    {
-      #ifdef EDITION_SE
-      const char * const type = "SE";
-      #else
-      const char * const type = "CE";
-      #endif
-      printf("MySQL Workbench %s %i.%i.%i %i\n"
-               , type
-               , APP_MAJOR_NUMBER
-               , APP_MINOR_NUMBER
-               , APP_RELEASE_NUMBER
-               , APP_REVISION_NUMBER
-            );
-
-      exit(0);
-    }
-    else if (strcmp(argv[i], "--verbose") == 0)
-      wboptions.verbose = true;
-    else if (strncmp(argv[i], "--log-level", 10) == 0)
-    {
-      if (!parse_loglevel(argv[i]) && (i+1) < argc) // If parse failed try to add next arg from CLI
-      {
-        std::string line(argv[i]);
-        line += argv[i + 1];
-
-        if (!parse_loglevel(line))
-        {
-          if ( (i + 2) < argc ) // Yet, we may have three CLI args if it was written like --log-level = <level>. Handle extra spaces
-          {
-            line += argv[i+2];
-            if (parse_loglevel(line))
-              i += 2; // correct arg count, so we do not parse log-level parts as smth different
-          }
-        }
-        else
-          ++i; // correct arg count, so we do not parse log-level parts as smth different
-      }
-    }
-    else if (*argv[i] != '-')
-    {
-      wboptions.open_at_startup= argv[i];
-    }
-    else
-    {
-      printf("Unknown option %s\n", argv[i]);
-      //show_help();
-      //exit(0);
-    }
-    i++;
-  }
+  if (!wboptions.parse_args(argv, argc))
+    return 1;
 
   if (getenv("WB_VERBOSE"))
     wboptions.verbose = true;
@@ -272,9 +137,7 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-
-
-  mforms::gtk::init();
+  mforms::gtk::init(getenv("WB_FORCE_SYSTEM_COLORS") != NULL);
   mforms::gtk::GRTTreeViewImpl::init();
   mforms::gtk::WizardImpl::set_icon_path(wboptions.basedir+"/images");
 
@@ -289,7 +152,7 @@ int main(int argc, char **argv)
     XSynchronize(gdk_display, 1);
   }
 
-  if (!wboptions.quit_when_done)
+  if (!wboptions.quit_when_done || wboptions.no_logo)
   {
     Gtk::Window *splash = show_splash();
     splash->get_window()->freeze_updates(); // Added to stop idle window update requests storm in gnome-3
