@@ -16,10 +16,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
+
 #include "stdafx.h"
 
-using namespace mforms;
+#include "mforms/mforms.h"
 
+using namespace mforms;
 
 Panel::Panel(PanelType type)
 {
@@ -28,6 +30,19 @@ Panel::Panel(PanelType type)
   _panel_impl->create(this, type);
 }
 
+Panel::~Panel()
+{
+  set_destroying();
+
+  std::list<std::pair<View*,bool> >::iterator iter;
+
+  while ((iter = _subviews.begin()) != _subviews.end())
+  {
+    View *sv = iter->first;
+    _panel_impl->remove(this, sv);
+    View::remove(sv);
+  }
+}
 
 void Panel::set_title(const std::string &title)
 {
@@ -54,13 +69,10 @@ bool Panel::get_active()
 
 
 void Panel::add(View *subview)
-{
-  subview->retain();
-  
-  subview->set_parent(this);
-  (*_panel_impl->add)(this, subview);
-  subview->show();
+{  
   add_subview(subview);
+  _panel_impl->add(this, subview);
+  subview->show();
   
   relayout();
 }
@@ -68,13 +80,9 @@ void Panel::add(View *subview)
 
 void Panel::remove(View *subview)
 {  
-  subview->set_parent(0);
-  remove_subview(subview);
-  (*_panel_impl->remove)(this, subview);
+  _panel_impl->remove(this, subview);
+  View::remove(subview);
  
-  if (subview)
-    subview->release();
-
   relayout();
 }
 
