@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,18 +23,25 @@
 
 #include "stdafx.h"
 
-#include "base/log.h"
+#include "mforms/mforms.h"
+#include "wf_view.h"
+#include "wf_box.h"
+#include "workbench/wb_context.h"
+#include "wf_appview.h"
 #include "wf_app.h"
+#include "wf_dockingpoint.h"
+
+#include "base/log.h"
+#include "mdc.h"
 #include "GrtTemplates.h"
 
 DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_WRAPPER)
 
 using namespace System::Threading;
+using namespace System::IO;
 
 using namespace MySQL::Forms;
 using namespace MySQL::Grt;
-using namespace MySQL::Geometry;
-using namespace MySQL::Drawing;
 
 using namespace base;
 
@@ -55,8 +62,9 @@ AppImpl::AppImpl(MySQL::Forms::Manager^ mgr,
   VoidRefreshTypeStringIntPtrDelegate^ refresh_gui,
   VoidBoolDelegate^ lock_gui,
   VoidStrDelegate^ perform_command,
-  VoidDelegate^ quit_application,
-  StringAppCommandAppViewStringDelegate^ app_command)
+  BoolDelegate^ quit_application,
+  StringAppCommandAppViewStringDelegate^ app_command,
+  DockingPointDelegateWrapper^ docking_delegate)
   : _callbacks(new ::wb::WBFrontendCallbacks())
 {
   Logger::log(Logger::LogDebug, DOMAIN_MFORMS_WRAPPER, "Creating application wrapper\n");
@@ -64,11 +72,14 @@ AppImpl::AppImpl(MySQL::Forms::Manager^ mgr,
   // Store a reference to this class in the backend.
   _cb_handle = GCHandle::Alloc(this);
   IntPtr pointer= GCHandle::ToIntPtr(_cb_handle);
+
+  _docking_delegate = docking_delegate;
+  mforms::App::instantiate(docking_delegate->get_unmanaged_delegate(), false);
+
   mforms::App::get()->set_data(pointer.ToPointer());
 
   set_show_file_dialog(show_file_dialog);
   set_show_status_text(show_status_text);
-  set_show_progress(show_progress);
   set_request_input(request_input);
 
   set_create_diagram(create_diagram);
@@ -96,6 +107,7 @@ AppImpl::AppImpl(MySQL::Forms::Manager^ mgr,
 
 AppImpl::~AppImpl()
 {
+//  delete _dock_delegate;
   _cb_handle.Free();
   delete _callbacks;
   Logger::log(Logger::LogDebug, DOMAIN_MFORMS_WRAPPER, "Destroyed backend application wrapper\n");
@@ -110,6 +122,24 @@ AppImpl::~AppImpl()
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Attempts to find the full path to the given image file. If the given path is already an absolute
+ * one then it is returned as is. Otherwise the C# frontend is queried to find the file in any of the
+ * (image) resource paths and if found a full path is constructed.
+ * Note: this is a pure platform function and hence requires no string conversion like the (similar)
+ *       get_resource_path function, which is used by the backend.
+ */
+String^ AppImpl::get_image_path(String ^path)
+{
+  if (Path::IsPathRooted(path))
+    return path;
+
+  AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) mforms::App::get()->get_data_ptr());
+  return  application->app_command_delegate(AppCommand::AppGetResourcePath, nullptr, path);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bool AppImpl::request_input_wrapper(const std::string &title, int flags, std::string& res)
 {
   String^ result= nullptr;
@@ -131,7 +161,7 @@ void AppImpl::set_show_file_dialog(StrStrStrStrDelegate^ dt)
     gcnew StrStrStrStrWrapperDelegate(this, &AppImpl::show_file_dialog_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(show_file_dialog_wrapper_delegate);
   STR_STR_STR_STR_CB cb = static_cast<STR_STR_STR_STR_CB>(ip.ToPointer());
-  _callbacks->show_file_dialog= boost::bind(cb, _1, _2, _3);
+  _callbacks->show_file_dialog= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -143,19 +173,7 @@ void AppImpl::set_show_status_text(VoidStrDelegate^ dt)
     gcnew VoidStrWrapperDelegate(this, &AppImpl::show_status_text_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(show_status_text_wrapper_delegate);
   VOID_STR_CB cb = static_cast<VOID_STR_CB>(ip.ToPointer());
-  _callbacks->show_status_text= boost::bind(cb, _1);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void AppImpl::set_show_progress(BoolStrStrFloatDelegate^ dt)
-{
-  show_progress_delegate= dt;
-  show_progress_wrapper_delegate= 
-    gcnew BoolStrStrFloatWrapperDelegate(this, &AppImpl::show_progress_wrapper);
-  IntPtr ip = Marshal::GetFunctionPointerForDelegate(show_progress_wrapper_delegate);
-  BOOL_STR_STR_FLOAT_CB cb = static_cast<BOOL_STR_STR_FLOAT_CB>(ip.ToPointer());
-  _callbacks->show_progress= boost::bind(cb, _1, _2, _3);
+  _callbacks->show_status_text= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -167,7 +185,7 @@ void AppImpl::set_request_input(BoolStrIntStrPtrDelegate^ dt)
     gcnew BoolStrIntStrPtrWrapperDelegate(this, &AppImpl::request_input_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(request_input_wrapper_delegate);
   BOOL_STR_INT_STRPTR_CB cb = static_cast<BOOL_STR_INT_STRPTR_CB>(ip.ToPointer());
-  _callbacks->request_input= boost::bind(cb, _1, _2, _3);
+  _callbacks->request_input= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -179,7 +197,7 @@ void AppImpl::set_create_diagram(CanvasViewStringStringIntPtrDelegate^ dt)
     gcnew CanvasViewDiagramWrapperDelegate(this, &AppImpl::create_diagram_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(create_diagram_wrapper_delegate);
   CANVASVIEW_DIAGRAM_CB cb = static_cast<CANVASVIEW_DIAGRAM_CB>(ip.ToPointer());
-  _callbacks->create_diagram= boost::bind(cb, _1);
+  _callbacks->create_diagram= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -191,7 +209,7 @@ void AppImpl::set_destroy_view(VoidCanvasViewDelegate^ dt)
     gcnew VoidCanvasViewWrapperDelegate(this, &AppImpl::destroy_view_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(destroy_view_wrapper_delegate);
   VOID_CANVASVIEW_CB cb = static_cast<VOID_CANVASVIEW_CB>(ip.ToPointer());
-  _callbacks->destroy_view= boost::bind(cb, _1);
+  _callbacks->destroy_view= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -203,7 +221,7 @@ void AppImpl::set_switched_view(VoidCanvasViewDelegate^ dt)
     gcnew VoidCanvasViewWrapperDelegate(this, &AppImpl::switched_view_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(switched_view_wrapper_delegate);
   VOID_CANVASVIEW_CB cb = static_cast<VOID_CANVASVIEW_CB>(ip.ToPointer());
-  _callbacks->switched_view= boost::bind(cb, _1);
+  _callbacks->switched_view= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -215,7 +233,7 @@ void AppImpl::set_tool_changed(VoidCanvasViewDelegate^ dt)
     gcnew VoidCanvasViewWrapperDelegate(this, &AppImpl::tool_changed_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(tool_changed_wrapper_delegate);
   VOID_CANVASVIEW_CB cb = static_cast<VOID_CANVASVIEW_CB>(ip.ToPointer());
-  _callbacks->tool_changed= boost::bind(cb, _1);
+  _callbacks->tool_changed= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -227,7 +245,7 @@ void AppImpl::set_open_editor(IntPtrGRTManagerModuleStrStrGrtListFlagsDelegate^
     gcnew IntGRTManagerModuleStrStrGrtListFlagsWrapperDelegate(this, &AppImpl::open_editor_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(open_editor_wrapper_delegate);
   INT_GRTMANAGER_MODULE_STR_STR_GRTLIST_FLAGS_CB cb = static_cast<INT_GRTMANAGER_MODULE_STR_STR_GRTLIST_FLAGS_CB>(ip.ToPointer());
-  _callbacks->open_editor= boost::bind(cb, _1, _2, _3, _4, _5, _6);
+  _callbacks->open_editor= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -239,7 +257,7 @@ void AppImpl::set_show_editor(VoidIntPtrDelegate^ dt)
     gcnew VoidIntPtrWrapperDelegate(this, &AppImpl::show_editor_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(show_editor_wrapper_delegate);
   VOID_INT64_CB cb = static_cast<VOID_INT64_CB>(ip.ToPointer());
-  _callbacks->show_editor= boost::bind(cb, _1);
+  _callbacks->show_editor= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -251,7 +269,7 @@ void AppImpl::set_hide_editor(VoidIntPtrDelegate^ dt)
     gcnew VoidIntPtrWrapperDelegate(this, &AppImpl::hide_editor_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(hide_editor_wrapper_delegate);
   VOID_INT64_CB cb = static_cast<VOID_INT64_CB>(ip.ToPointer());
-  _callbacks->hide_editor= boost::bind(cb, _1);
+  _callbacks->hide_editor= cb;
 }
 
 
@@ -264,7 +282,7 @@ void AppImpl::set_refresh_gui(VoidRefreshTypeStringIntPtrDelegate ^ dt)
     gcnew VoidRefreshTypeStringIntPtrWrapperDelegate(this, &AppImpl::refresh_gui_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(refresh_gui_wrapper_delegate);
   VOID_REFRESHTYPE_STRING_INTPTR_CB cb = static_cast<VOID_REFRESHTYPE_STRING_INTPTR_CB>(ip.ToPointer());
-  _callbacks->refresh_gui= boost::bind(cb, _1, _2, _3);
+  _callbacks->refresh_gui= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -276,7 +294,7 @@ void AppImpl::set_perform_command(VoidStrDelegate ^ dt)
     gcnew VoidStrWrapperDelegate(this, &AppImpl::perform_command_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(perform_command_wrapper_delegate);
   VOID_STR_CB cb = static_cast<VOID_STR_CB>(ip.ToPointer());
-  _callbacks->perform_command = boost::bind(cb, _1);
+  _callbacks->perform_command = cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -288,19 +306,19 @@ void AppImpl::set_lock_gui(VoidBoolDelegate^ dt)
     gcnew VoidBoolWrapperDelegate(this, &AppImpl::lock_gui_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(lock_gui_wrapper_delegate);
   VOID_BOOL_CB cb = static_cast<VOID_BOOL_CB>(ip.ToPointer());
-  _callbacks->lock_gui= boost::bind(cb, _1);
+  _callbacks->lock_gui= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void AppImpl::set_quit_application(VoidDelegate^ dt)
+void AppImpl::set_quit_application(BoolDelegate^ dt)
 {
   quit_application_delegate= dt;
   quit_application_wrapper_delegate=
     gcnew VoidWrapperDelegate(this, &AppImpl::quit_application_wrapper);
   IntPtr ip = Marshal::GetFunctionPointerForDelegate(quit_application_wrapper_delegate);
-  VOID_CB cb = static_cast<VOID_CB>(ip.ToPointer());
-  _callbacks->quit_application= boost::bind(cb);
+  BOOL_CB cb = static_cast<BOOL_CB>(ip.ToPointer());
+  _callbacks->quit_application= cb;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -443,38 +461,6 @@ Object^ AppImpl::GetFromFixedId(IntPtr ip)
 
 //--------------------------------------------------------------------------------------------------
 
-void AppImpl::dock_view(mforms::App *app, mforms::AppView *view, const std::string &position)
-{
-  AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) app->get_data_ptr());
-  application->app_command_wrapper(AppCommand::AppDockView, view, position);
-}
-
-//--------------------------------------------------------------------------------------------------
-
-bool AppImpl::select_view(mforms::App *app, const std::string &identifier)
-{
-  AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) app->get_data_ptr());
-  return !application->app_command_wrapper(AppCommand::AppSelectView, nullptr, identifier).empty();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void AppImpl::undock_view(mforms::App *app, mforms::AppView *view)
-{
-  AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) app->get_data_ptr());
-  application->app_command_wrapper(AppCommand::AppUndockView, view, "");
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void AppImpl::set_view_title(mforms::App *app, mforms::AppView *view, const std::string &title)
-{
-  AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) app->get_data_ptr());
-  application->app_command_wrapper(AppCommand::AppSetViewTitle, view, title);
-}
-
-//--------------------------------------------------------------------------------------------------
-
 std::string AppImpl::get_resource_path(mforms::App *app, const std::string &file)
 {
   AppImpl^ application= (AppImpl^) GetFromFixedId((IntPtr) app->get_data_ptr());
@@ -535,24 +521,40 @@ void AppImpl::exit_event_loop(mforms::App *app, int ret_code)
 
 //--------------------------------------------------------------------------------------------------
 
-MySQL::Drawing::Color AppImpl::get_system_color(mforms::SystemColor type)
+base::Color AppImpl::get_system_color(mforms::SystemColor type)
 {
+  System::Drawing::Color^ color;
   switch (type)
   {
-  case mforms::SystemHighlight:
-    {
-      System::Drawing::Color^ color = System::Drawing::Color::FromKnownColor(KnownColor::Highlight);
-      return NativeToColor(color);
-    }
+  case mforms::SystemColorHighlight:
+      color = System::Drawing::Color::FromKnownColor(KnownColor::Highlight);
+      break;
+  case mforms::SystemColorEditor:
+      color = System::Drawing::Color::FromKnownColor(KnownColor::Window);
+      break;
+  case mforms::SystemColorDisabled:
+      color = System::Drawing::Color::FromKnownColor(KnownColor::ButtonFace);
+      break;
+  case mforms::SystemColorContainer:
+    color = System::Drawing::Color::FromKnownColor(KnownColor::Control);
+    break;
   default:
 #ifdef _DEBUG
     throw new std::runtime_error("mforms::App: Invalid system color enumeration given.");
 #else
     log_error("App: Invalid system color enumeration given.\n");
-    return MySQL::Drawing::Color::Black();
+    return base::Color::Black();
 #endif
   }
+
+  return NativeToColor(color);
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void AppImpl::close_splash(mforms::App *app)
+{
+  MySQL::Utilities::SplashScreen::CloseSplash();
+}
+
+//--------------------------------------------------------------------------------------------------
