@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,12 +27,11 @@
 #include <typeinfo>
 #include <list>
 #include <map>
-#include <deque>
+//#include <deque>
 #include <vector>
 #include <stdexcept>
 #include <boost/function.hpp>
 #include <libxml/xmlmemory.h>
-#include <boost/signals2.hpp>
  		  	
 #include <glib.h>
 
@@ -136,7 +135,9 @@ namespace grt {
   class MYSQLGRT_PUBLIC module_error : public std::runtime_error
   {
   public:
-    module_error(const std::string &exc) : std::runtime_error(exc) {}
+    std::string inner;
+    module_error(const std::string &exc, const std::string &ainner = "") : std::runtime_error(exc), inner(ainner) {}
+    virtual ~module_error() THROW() {}
   };
 
   class MYSQLGRT_PUBLIC user_cancelled : public std::runtime_error
@@ -144,6 +145,18 @@ namespace grt {
   public:
     user_cancelled(const std::string &exc) : std::runtime_error(exc) {}
   };  
+  
+  class MYSQLGRT_PUBLIC db_login_error : public std::runtime_error
+  {
+  public:
+    db_login_error(const std::string &exc) : std::runtime_error(exc) {}
+  };
+  
+  class MYSQLGRT_PUBLIC db_access_denied : public std::runtime_error
+  {
+  public:
+    db_access_denied(const std::string &exc) : std::runtime_error(exc) {}
+  };
 };
 
 //------------------------------------------------------------------------------------------------
@@ -259,6 +272,9 @@ namespace grt {
     int refcount() const { return _value->refcount(); }
     void retain() { if (_value) _value->retain(); }
     void release() { if (_value)_value->release(); }
+
+    void mark_global() const {if (_value)_value->mark_global();}
+    void unmark_global() const {if (_value)_value->unmark_global();}
      
   protected:
     internal::Value *_value;
@@ -341,7 +357,7 @@ namespace grt {
     template<class Subclass>
       Ref(const Ref<Subclass> &ref)
     {
-#ifdef ENABLE_DEBUG
+#if defined(ENABLE_DEBUG) || defined(_DEBUG)
       // poor mans compile-time type "check". compiler will throw an error if Subclass is not derived from Class
 #ifdef __GNUC__
       Class *dummy_variable_just_for_type_check __attribute__((unused))= static_cast<Subclass*>(ref.valueptr());
@@ -410,6 +426,7 @@ namespace grt {
     Class &content() const { return *static_cast<Class*>(_value); }
   };
 
+#if 0
   //----------------------------------------------------------------------
   // Weak Object Refs
   
@@ -513,7 +530,7 @@ namespace grt {
       Class *_content;
       internal::ObjectValidFlag _valid_flag;
     };
-    
+#endif // disabled
   //----------------------------------------------------------------------
   // IntegerRef
 
@@ -1839,6 +1856,8 @@ namespace grt {
   class PropertyBase
   {
   public:
+    virtual ~PropertyBase() {};
+    
     virtual bool has_setter() const= 0;
        
     virtual void set(internal::Object *obj, const grt::ValueRef &value)= 0;
@@ -2390,7 +2409,7 @@ namespace grt {
     std::string format(bool withtype= false) const;
   };
 
-  typedef boost::function<void (const Message&, void*)> MessageSlot;
+  typedef boost::function<bool (const Message&, void*)> MessageSlot;
   typedef boost::function<bool ()> StatusQuerySlot;
 
   //-----------------------------------------------------------------------------------------------
@@ -2559,6 +2578,8 @@ namespace grt {
     
     const std::vector<Module*> &get_modules() const { return _modules; }
     
+    grt::ValueRef call_module_function(const std::string &module, const std::string &function, const grt::BaseListRef &args);
+    
     Module *get_module(const std::string &name);
     
     // create an instance of the given native module and registers it with the GRT.
@@ -2684,8 +2705,13 @@ namespace grt {
 
 
     // grt logging/messaging
-    MessageSlot set_message_handler(const MessageSlot &slot);
-    StatusQuerySlot set_status_query_handler(const StatusQuerySlot &slot);
+    int message_handler_count() { return (int)_message_slot_stack.size(); }
+    void push_message_handler(const MessageSlot &slot);
+    void pop_message_handler();
+    
+    void push_status_query_handler(const StatusQuerySlot &slot);
+    void pop_status_query_handler();
+    bool query_status();
      
     void make_output_visible(void *sender= NULL);
 
@@ -2693,6 +2719,9 @@ namespace grt {
     void send_warning(const std::string &message, const std::string &details="", void *sender= NULL);
     void send_info(const std::string &message, const std::string &details="", void *sender= NULL);
     void send_progress(float percentage, const std::string &message, const std::string &details="", void *sender= NULL);
+    void begin_progress_step(float from, float to);
+    void end_progress_step();
+    void reset_progress_steps();
 
     void send_verbose(const std::string &message, void *sender= NULL);
     void send_output(const std::string &text, void *sender= NULL);
@@ -2712,8 +2741,10 @@ namespace grt {
 
     //std::map<std::string,internal::Object*> _objects;
 
-    MessageSlot _message_slot;
-    StatusQuerySlot _status_query_slot;
+    std::vector<MessageSlot> _message_slot_stack;
+    std::vector<StatusQuerySlot> _status_query_slot_stack;
+    
+    std::vector<std::pair<float,float> > _progress_step_stack;
 
     GStaticRecMutex _message_mutex;
 
@@ -2733,6 +2764,8 @@ namespace grt {
 
     void add_metaclass(MetaClass *stru);
     std::string module_path_in_bundle(const std::string &path);
+    
+    bool handle_message(const Message &msg, void *sender);
 
     std::map<std::string,MetaClass*> _metaclasses;
     std::list<MetaClass*> _metaclasses_list;
