@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@
 #include "mysql_sql_parser_fe.h"
 #include "base/string_utilities.h"
 #include <boost/foreach.hpp>
-#include <boost/lambda/lambda.hpp>
+#include <boost/lambda/bind.hpp>
 
 using namespace grt;
 using namespace base;
@@ -33,8 +33,7 @@ using namespace base;
 
 Mysql_sql_statement_decomposer::Null_state_keeper::~Null_state_keeper()
 {
-    static const Parse_result pr = pr_irrelevant;
-    boost::function<Parse_result ()> f = boost::lambda::var(pr);
+    boost::function<Parse_result ()> f = boost::lambda::constant(pr_irrelevant);
     _sql_parser->_do_process_sql_statement = boost::bind(f);
 }
 #define NULL_STATE_KEEPER Null_state_keeper _nsk(this);
@@ -472,7 +471,7 @@ void Mysql_sql_statement_decomposer::expand_wildcards(SelectStatement::Ref selec
                   db_ViewRef db_view= find_named_object_in_list(db_schema->views(), table, _case_sensitive_identifiers);
                   if (db_view.is_valid())
                   {
-                    GRTLIST_FOREACH (grt::internal::String, db_view->columns(), col)
+                    for(grt::StringListRef::const_iterator col = db_view->columns().begin(); col != db_view->columns().end(); ++col)
                     {
                       SelectItem new_select_item;
                       new_select_item.schema= schema;
