@@ -34,6 +34,7 @@
 #include <ctype.h>
 
 #include "module_db_mysql.h"
+#include "module_db_mysql_shared_code.h"
 
 void DiffSQLGeneratorBE::remember(const GrtNamedObjectRef &obj, const std::string &sql)
 {
@@ -45,7 +46,8 @@ void DiffSQLGeneratorBE::remember(const GrtNamedObjectRef &obj, const std::strin
   }
   else
   {
-    target_map.set(get_full_object_name_for_key(obj,_case_sensitive), grt::StringRef(sql));
+    target_map.set(_use_oid_as_dict_key ? obj.id() : get_full_object_name_for_key(obj,_case_sensitive), 
+                   grt::StringRef(sql));
   }
 }
 
@@ -59,7 +61,7 @@ void DiffSQLGeneratorBE::remember_alter(const GrtNamedObjectRef &obj, const std:
     return;
   }
 
-  std::string key= get_full_object_name_for_key(obj,_case_sensitive);
+  std::string key= _use_oid_as_dict_key ? obj.id() : get_full_object_name_for_key(obj,_case_sensitive);
 
   if(target_map.has_key(key))
   {
@@ -88,13 +90,6 @@ void DiffSQLGeneratorBE::remember_alter(const GrtNamedObjectRef &obj, const std:
 }
 
 
-struct Part_sort_pred {
-  bool operator() (const std::pair<int, grt::ValueRef>& v1, const std::pair<int, grt::ValueRef>& v2)
-  {
-    return v1 < v2;
-  }
-};
-
 void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table, 
                                                    const grt::DiffChange *table_diffchange)
 {
@@ -114,47 +109,44 @@ void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table,
     if(attr_change->get_attr_name().compare("partitionType") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       part_type.assign(grt::StringRef::cast_from(change->get_new_value()));
       part_type_set= true;
     }
     else if(attr_change->get_attr_name().compare("partitionExpression") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       part_expr.assign(grt::StringRef::cast_from(change->get_new_value()));
       part_expr_set= true;
     }
     else if(attr_change->get_attr_name().compare("subpartitionType") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       subpart_type.assign(grt::StringRef::cast_from(change->get_new_value()));
       subpart_type_set= true;
     }
     else if(attr_change->get_attr_name().compare("subpartitionExpression") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       subpart_expr.assign(grt::StringRef::cast_from(change->get_new_value()));
       subpart_expr_set= true;
     }
     else if(attr_change->get_attr_name().compare("partitionCount") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       part_count= grt::IntegerRef::cast_from(change->get_new_value());
       part_count_set= true;
     }
     else if(attr_change->get_attr_name().compare("partitionDefinitions") == 0)
     {
       const grt::MultiChange *change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *part_defs_cs= change->subchanges();
 
-      // needs to be sorted by LESS THAN value
-      std::vector<std::pair<int, grt::ValueRef> > partition_vector;
-
       for(grt::ChangeSet::const_iterator part_defs_e= part_defs_cs->end(), 
         part_defs_it= part_defs_cs->begin(); part_defs_it != part_defs_e; part_defs_it++)
       {
@@ -179,19 +171,8 @@ void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table,
         db_mysql_PartitionDefinitionRef part_def_value(db_mysql_PartitionDefinitionRef::cast_from(part_def_grt_value));
 
         if (part_def_value.is_valid())
-          partition_vector.push_back(std::pair<int, grt::ValueRef>(
-            atoi(part_def_value->value().c_str()), part_def_value));
+            part_defs.insert(part_def_value);
       }
-
-
-      std::sort(partition_vector.begin(), partition_vector.end());
-
-      for(std::vector<std::pair<int, grt::ValueRef> >::const_iterator pv_e= partition_vector.end(), 
-        pv_it= partition_vector.begin(); pv_it != pv_e; pv_it++)
-      {
-        part_defs.insert(db_mysql_PartitionDefinitionRef::cast_from(pv_it->second));
-      }
-
       part_defs_set= true;
     }
   }
@@ -326,13 +307,13 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(strlen(table->avgRowLength().c_str()))
       callback->create_table_avg_row_length(table->avgRowLength());
 
-    if(strlen(table->defaultCharacterSetName().c_str()))
+    if(table->defaultCharacterSetName().is_valid() && strlen(table->defaultCharacterSetName().c_str()))
       callback->create_table_charset(table->defaultCharacterSetName());
 
     if(table->checksum() != 0)
       callback->create_table_checksum(table->checksum());
 
-    if(strlen(table->defaultCollationName().c_str()))
+    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str()))
       callback->create_table_collate(table->defaultCollationName());
 
     if(strlen(table->comment().c_str()))
@@ -485,11 +466,8 @@ void DiffSQLGeneratorBE::generate_drop_stmt(db_mysql_TableRef table)
     return;
 
   std::string table_name_for_filter(get_old_object_name_for_key(table, _case_sensitive));
-  if(_use_filtered_lists)
-    if(_filtered_tables.find(table_name_for_filter) == _filtered_tables.end())
-      return;
-
-  callback->drop_table(table);
+  if (!(_use_filtered_lists && (_filtered_tables.find(table_name_for_filter) == _filtered_tables.end())))
+    callback->drop_table(table);
 
   grt::ListRef<db_mysql_Trigger> triggers= table->triggers();
   for(size_t c= triggers.count(), i= 0; i < c; i++)
@@ -537,7 +515,7 @@ void DiffSQLGeneratorBE::generate_drop_stmt(db_UserRef user)
 void DiffSQLGeneratorBE::generate_drop_stmt(db_mysql_SchemaRef schema)
 {
   callback->drop_schema(schema);
-
+  callback->disable_list_insert(true);
   grt::ListRef<db_mysql_Table> tables= schema->tables();
   for(size_t count= tables.count(), i= 0; i < count; i++)
   {
@@ -558,6 +536,8 @@ void DiffSQLGeneratorBE::generate_drop_stmt(db_mysql_SchemaRef schema)
     db_mysql_RoutineRef routine= routines.get(i);
     generate_drop_stmt(routine);
   }
+  callback->disable_list_insert(false);
+
 }
 
 void DiffSQLGeneratorBE::generate_drop_stmt(db_mysql_CatalogRef catalog)
@@ -626,22 +606,6 @@ void DiffSQLGeneratorBE::generate_alter(grt::ListRef<db_mysql_Column> columns, c
     callback->alter_table_drop_column(table, db_mysql_ColumnRef::cast_from(removed_change->get_value()));
   }
 
-  // process ADD COLUMN
-  for(grt::ChangeSet::const_iterator e= columns_cs->end(), it= columns_cs->begin(); it != e; it++)
-  {
-    const boost::shared_ptr<grt::DiffChange> column_change= *it;
-    if(column_change->get_change_type() != grt::ListItemAdded)
-      continue;
-
-    const grt::ListItemAddedChange *added_change= static_cast<const grt::ListItemAddedChange *>(column_change.get());
-
-    callback->alter_table_add_column(
-      table, 
-      column_rename_map,
-      db_mysql_ColumnRef::cast_from(grt::ValueRef(added_change->get_value())),
-      db_mysql_ColumnRef::cast_from(added_change->get_prev_item()));
-  }
-
   std::set<grt::ValueRef> processed_items;
 
   // process CHANGE COLUMN (handles both position and content change)
@@ -686,6 +650,22 @@ void DiffSQLGeneratorBE::generate_alter(grt::ListRef<db_mysql_Column> columns, c
     );
   }
 
+    // process ADD COLUMN
+  for(grt::ChangeSet::const_iterator e= columns_cs->end(), it= columns_cs->begin(); it != e; it++)
+  {
+    const boost::shared_ptr<grt::DiffChange> column_change= *it;
+    if(column_change->get_change_type() != grt::ListItemAdded)
+      continue;
+
+    const grt::ListItemAddedChange *added_change= static_cast<const grt::ListItemAddedChange *>(column_change.get());
+
+    callback->alter_table_add_column(
+      table, 
+      column_rename_map,
+      db_mysql_ColumnRef::cast_from(grt::ValueRef(added_change->get_value())),
+      db_mysql_ColumnRef::cast_from(added_change->get_prev_item()));
+  }
+
   callback->alter_table_columns_end(table);
 }
 
@@ -890,8 +870,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt_drops(db_mysql_TableRef table, cons
   if(!process_table)
     return;
   const grt::ChangeSet* cs= diffchange->subchanges();
-
-  callback->alter_table_props_begin(table);
+  bool gen_alter = false;
 
   for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); it != e; it++)
   {
@@ -901,15 +880,20 @@ void DiffSQLGeneratorBE::generate_alter_stmt_drops(db_mysql_TableRef table, cons
     if(attr_change->get_attr_name().compare("foreignKeys") == 0)
     {
       const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
+      if (!gen_alter)
+      {
+          gen_alter = true;
+          callback->alter_table_props_begin(table);      
+      }
 
       callback->alter_table_fks_begin(table);
       generate_alter_drop(table->foreignKeys(), list_change);
       callback->alter_table_fks_end(table);
     }
   }
-
-  callback->alter_table_props_end(table);
+  if (gen_alter)
+    callback->alter_table_props_end(table);
 
 }
 
@@ -936,7 +920,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     if(attr_change->get_attr_name().compare("triggers") == 0)
     {
       const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       
       const grt::ChangeSet *triggers_cs= list_change->subchanges();
       
@@ -996,20 +980,20 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     if(attr_change->get_attr_name().compare("name") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_name(table, str);
     }
     if(attr_change->get_attr_name().compare("columns") == 0)
     {
       const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       generate_alter(table->columns(), list_change);
     }
     else if(attr_change->get_attr_name().compare("indices") == 0)
     {
       const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       callback->alter_table_indexes_begin(table);
       generate_alter(table->indices(), list_change);
       callback->alter_table_indexes_end(table);
@@ -1017,7 +1001,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     else if(attr_change->get_attr_name().compare("foreignKeys") == 0)
     {
       const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       
       callback->alter_table_fks_begin(table);
       generate_alter(table->foreignKeys(), list_change);
@@ -1026,56 +1010,57 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     else if(attr_change->get_attr_name().compare("tableEngine") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
-      callback->alter_table_engine(table, str);
+      if(!str.empty())
+        callback->alter_table_engine(table, str);
     }
     else if(attr_change->get_attr_name().compare("nextAutoInc") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_next_auto_inc(table, str);
     }
     else if(attr_change->get_attr_name().compare("password") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_password(table, str);
     }
     else if(attr_change->get_attr_name().compare("delayKeyWrite") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::IntegerRef n= grt::IntegerRef::cast_from(change->get_new_value());
       callback->alter_table_delay_key_write(table, n);
     }
     else if(attr_change->get_attr_name().compare("defaultCharacterSetName") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_charset(table, str);
     }
     else if(attr_change->get_attr_name().compare("defaultCollationName") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_collate(table, str);
     }
     else if(attr_change->get_attr_name().compare("mergeUnion") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_merge_union(table, str);
     }
     else if(attr_change->get_attr_name().compare("mergeInsert") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_merge_insert(table, str);
     }
@@ -1083,13 +1068,13 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
 #if 0
     else if(attr_change->get_attr_name().compare("tableDataDir") == 0)
     {
-      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       alter_sql.append("DATA DIRECTORY = '").append(str.c_str()).append("' ");
     }
     else if(attr_change->get_attr_name().compare("tableIndexDir") == 0)
     {
-      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       alter_sql.append("INDEX DIRECTORY = '").append(str.c_str()).append("' ");
     }
@@ -1097,48 +1082,48 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     else if(attr_change->get_attr_name().compare("packKeys") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_pack_keys(table, str);
     }
     else if(attr_change->get_attr_name().compare("checksum") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::IntegerRef n= grt::IntegerRef::cast_from(change->get_new_value());
       callback->alter_table_checksum(table, n);
     }
     else if(attr_change->get_attr_name().compare("comment") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_comment(table, str);
     }
     else if(attr_change->get_attr_name().compare("rowFormat") == 0)
     {
-      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_row_format(table, str);
     }
     else if(attr_change->get_attr_name().compare("avgRowLength") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_avg_row_length(table, str);
     }
     else if(attr_change->get_attr_name().compare("minRows") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_min_rows(table, str);
     }
     else if(attr_change->get_attr_name().compare("maxRows") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_max_rows(table, str);
     }
@@ -1151,7 +1136,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     else if(attr_change->get_attr_name().compare("connectionString") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_connection_string(table, str);
     }
@@ -1164,15 +1149,10 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       if(partitions_processed)
         continue;
 
-      const grt::SimpleValueChange *change=
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
-      grt::StringRef new_str= grt::StringRef::cast_from(change->get_new_value());
-      grt::StringRef old_str= grt::StringRef::cast_from(change->get_old_value());
-
-      if ((strlen(new_str.c_str()) == 0) && ((attr_change->get_attr_name().compare("partitionExpression") != 0))) // drop partitioning
-        callback->alter_table_drop_partitioning(table);
-      else  // add/change partitioning
-        generate_set_partitioning(table, diffchange);
+      // TODO: is this get call still needed (no side effects) or can we remove it?
+/*unused      const grt::SimpleValueChange *change= */
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
+      generate_set_partitioning(table, diffchange);
 
       partitions_processed= true;
     }
@@ -1194,8 +1174,8 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     if(attr_change->get_attr_name().compare("partitionCount") == 0)
     {
       const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
-      grt::IntegerRef new_count= grt::IntegerRef::cast_from(change->get_new_value());
+        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
+      grt::IntegerRef old_count= grt::IntegerRef::cast_from(change->get_old_value());
       
       //std::string part_count_sql(generate_change_partition_count(table, new_count));
 
@@ -1214,13 +1194,13 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       if((part_type.find("HASH") != std::string::npos)
           || (part_type.find("KEY") != std::string::npos))
       {
-        callback->alter_table_partition_count(table, new_count);
+        callback->alter_table_partition_count(table, old_count);
         partitions_processed= true;
       }
     }
     else if(attr_change->get_attr_name().compare("partitionDefinitions") == 0)
     {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *part_cs= list_change->subchanges();
       for(grt::ChangeSet::const_iterator e2= part_cs->end(), jt= part_cs->begin(); jt != e2; jt++)
       {
@@ -1272,7 +1252,9 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_ViewRef old_view, db_mysql
 
   generate_create_stmt(new_view);
 
-  if(strcmp(old_view->name().c_str(), new_view->name().c_str())) // name changed - need to drop old view
+  std::string new_view_name = _case_sensitive ? *new_view->name() : base::toupper(new_view->name());
+  std::string old_view_name = _case_sensitive ? *old_view->name() : base::toupper(old_view->name());
+  if(strcmp(new_view_name.c_str(), old_view_name.c_str())) // name changed - need to drop old view
     generate_drop_stmt(old_view);
 }
 
@@ -1297,7 +1279,6 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     if(_filtered_schemata.find(schema_name_for_filter) == _filtered_schemata.end())
       process_alter_schema= false;
 
-  std::string alter_sql;
   const grt::ChangeSet *cs= diffchange->subchanges();
 
   callback->alter_schema_props_begin(schema);
@@ -1311,18 +1292,18 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
       if(attr_change->get_attr_name().compare("name") == 0)
       {
         const grt::SimpleValueChange *change= 
-          static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange());
+          static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
         grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
         callback->alter_schema_name(schema, str);
       }
       if(attr_change->get_attr_name().compare("defaultCharacterSetName") == 0)
       {
-        grt::ValueRef value= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange())->get_new_value();
+        grt::ValueRef value= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get())->get_new_value();
         callback->alter_schema_default_charset(schema, grt::StringRef::cast_from(value));
       }
       else if(attr_change->get_attr_name().compare("defaultCollationName") == 0)
       {
-        grt::ValueRef value= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange())->get_new_value();
+        grt::ValueRef value= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get())->get_new_value();
         callback->alter_schema_default_collate(schema, grt::StringRef::cast_from(value));
       }
     }
@@ -1335,7 +1316,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     const grt::ObjectAttrModifiedChange *attr_change= static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
     if(attr_change->get_attr_name().compare("tables") == 0)
     {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *tables_cs= list_change->subchanges();
       for(grt::ChangeSet::const_iterator e2= tables_cs->end(), jt= tables_cs->begin(); jt != e2; jt++)
       {
@@ -1344,16 +1325,13 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
         {
           generate_alter_stmt_drops(
             db_mysql_TableRef::cast_from(static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()),
-            table_change->subchanges()->begin()->get());
+            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get());
         }else if (table_change->get_change_type() == grt::ListItemOrderChanged)
         {
             const grt::ListItemOrderChange *oc= 
               static_cast<const grt::ListItemOrderChange *>(table_change);
-            const grt::ListItemModifiedChange *mc= NULL;
-            if(!oc->subchanges()->empty())
-                mc= static_cast<const grt::ListItemModifiedChange *>(oc->subchanges()->begin()->get());
-            if(mc)
-              generate_alter_stmt_drops(db_mysql_TableRef::cast_from(mc->get_new_value()), mc->subchanges()->begin()->get());
+            if(oc->get_subchange())
+              generate_alter_stmt_drops(db_mysql_TableRef::cast_from(oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get());
         }
       }
     }
@@ -1365,7 +1343,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     const grt::ObjectAttrModifiedChange *attr_change= static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
     if(attr_change->get_attr_name().compare("tables") == 0)
     {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *tables_cs= list_change->subchanges();
       for(grt::ChangeSet::const_iterator e2= tables_cs->end(), jt= tables_cs->begin(); jt != e2; jt++)
       {
@@ -1383,17 +1361,14 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
         case grt::ListItemModified:
           generate_alter_stmt(db_mysql_TableRef::cast_from(
             static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()), 
-            table_change->subchanges()->begin()->get());
+            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get());
           break;
         case grt::ListItemOrderChanged:
           {
             const grt::ListItemOrderChange *oc= 
               static_cast<const grt::ListItemOrderChange *>(table_change);
-            const grt::ListItemModifiedChange *mc= NULL;
-            if(!oc->subchanges()->empty())
-              mc= static_cast<const grt::ListItemModifiedChange *>(oc->subchanges()->begin()->get());
-            if(mc)
-                generate_alter_stmt(db_mysql_TableRef::cast_from(mc->get_new_value()), mc->subchanges()->begin()->get());
+            if(oc->get_subchange())
+                generate_alter_stmt(db_mysql_TableRef::cast_from(oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get());
           }
           break;
           default: 
@@ -1403,7 +1378,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     }
     else if(attr_change->get_attr_name().compare("views") == 0)
     {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *views_cs= list_change->subchanges();
       for(grt::ChangeSet::const_iterator e2= views_cs->end(), jt= views_cs->begin(); jt != e2; jt++)
       {
@@ -1438,7 +1413,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     }
     else if(attr_change->get_attr_name().compare("routines") == 0)
     {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange());
+      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *routines_cs= list_change->subchanges();
       for(grt::ChangeSet::const_iterator e2= routines_cs->end(), jt= routines_cs->begin(); jt != e2; jt++)
       {
@@ -1500,7 +1475,7 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_CatalogRef catalog, const
 
       if(objattrchange->get_attr_name().compare("schemata") == 0)
       {
-        const grt::DiffChange *objattr_subchange= objattrchange->get_subchange();
+        const grt::DiffChange *objattr_subchange= objattrchange->get_subchange().get();
         if(objattr_subchange->get_change_type() == grt::ListModified)
         {
           const grt::MultiChange *schemata_list_change=
@@ -1522,18 +1497,16 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_CatalogRef catalog, const
               break;
             case grt::ListItemModified:
               generate_alter_stmt(db_mysql_SchemaRef::cast_from(
-                static_cast<const grt::ListItemModifiedChange *>(schema_subchange)->get_new_value()), schema_subchange->subchanges()->begin()->get());
+                static_cast<const grt::ListItemModifiedChange *>(schema_subchange)->get_new_value()), 
+                static_cast<const grt::ListItemModifiedChange *>(schema_subchange)->get_subchange().get());
               break;
             case grt::ListItemOrderChanged:
               {
                 const grt::ListItemOrderChange *oc= 
                   static_cast<const grt::ListItemOrderChange *>(schema_subchange);
-                const grt::ListItemModifiedChange *mc= NULL;
-                if(!oc->subchanges()->empty())
-                  mc= static_cast<const grt::ListItemModifiedChange *>(oc->subchanges()->begin()->get());
-                if(mc)
+                if(oc->get_subchange())
                   generate_alter_stmt(db_mysql_SchemaRef::cast_from(
-                    mc->get_new_value()), mc->subchanges()->begin()->get());
+                    oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get());
               }
               break;
               default: 
@@ -1553,10 +1526,13 @@ static void fill_set_from_list(grt::StringListRef string_list, std::set<std::str
 }
 
 DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEActionInterface *cb)
-  : padding(2), callback(cb), _gen_create_index(false), _use_filtered_lists(true), _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false)
+  : callback(cb), _gen_create_index(false), _use_filtered_lists(true), 
+   _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false)
 {
-  if(options.is_valid())
+  if (options.is_valid())
   {
+    if (options.has_key("UseOIDAsResultDictKey"))
+      _use_oid_as_dict_key = options.get_int("UseOIDAsResultDictKey", 0) != 0;
 
     if(options.has_key("SkipForeignKeys"))
     {
@@ -1575,7 +1551,7 @@ DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEA
       if(use_filtered_lists.is_valid())
         _use_filtered_lists= (use_filtered_lists.operator long() != 0);
     }
-	cb->set_short_names(false);
+    cb->set_short_names(false);
     if(options.has_key("UseShortNames"))
     {
       grt::IntegerRef use_short_names = grt::IntegerRef::cast_from(options.get("UseShortNames"));
@@ -1594,7 +1570,6 @@ DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEA
       if(gen_index_value.is_valid())
         _gen_create_index= (gen_index_value.operator long() != 0);
     }
-
     if(options.has_key("UserFilterList"))
     {
       grt::StringListRef user_list= grt::StringListRef::cast_from(options.get("UserFilterList"));
@@ -1636,6 +1611,15 @@ DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEA
       if(triggers_list.is_valid())
         fill_set_from_list(triggers_list, _filtered_triggers);
     }
+    
+    grt::DictRef db_settings = grt::DictRef::cast_from(options.get("DBSettings"));
+    if(db_settings.is_valid() && db_settings.has_key("CaseSensitive"))
+    {
+      grt::IntegerRef case_sensitive = grt::IntegerRef::cast_from(db_settings.get("CaseSensitive"));
+      if(case_sensitive.is_valid())
+        _case_sensitive = (case_sensitive.operator long() != 0);
+    }
+
   }
 }
 
