@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,15 +21,20 @@
 
 #include <glib/gstdio.h>
 
-#include "base/util_functions.h"
 #include "wb_model.h"
 #include "reporting.h"
 
-#include "grti/sqlgenerator.h"
+#include "interfaces/sqlgenerator.h"
 #include "grts/structs.workbench.h"
 
 #include "reporting_template_variables.h"
+
+#include "base/util_functions.h"
 #include "base/string_utilities.h"
+#include "base/geometry.h"
+#include "base/log.h"
+
+#include "mforms/code_editor.h"
 
 // Support for syntax highlighting in SQL output.
 #ifdef _WIN32
@@ -47,14 +52,14 @@
 #endif
 #include "SciLexer.h"
 
-#include "mysql_sql_editor_be.h"
-
 using namespace std;
 using namespace base;
 
 using ctemplate::Template;
 using ctemplate::TemplateDictionary;
 
+DEFAULT_LOG_DOMAIN("Reporting")
+
 //----------------- LexerDocument ------------------------------------------------------------------
 
 LexerDocument::LexerDocument(const std::string& text)
@@ -676,7 +681,8 @@ const Scintilla::LexerModule* setup_syntax_highlighter(db_mgmt_RdbmsRef rdbms)
 
   if (result != NULL)
   {
-    Mysql_sql_editor editor(rdbms);
+    mforms::CodeEditorConfig config(mforms::LanguageMySQL);
+    std::map<std::string, std::string> keywords = config.get_keywords();
     
     // Create the keyword lists used for lexing.
 	  for (int i = 0; i <= KEYWORDSET_MAX; i++)
@@ -685,11 +691,12 @@ const Scintilla::LexerModule* setup_syntax_highlighter(db_mgmt_RdbmsRef rdbms)
     
     // There are no predefined constants for the indices below, but the occupancy of the list array
     // can be seen in LexMySQL.cxx.
-    ((SCI_WRAPPER_NS WordList*) keywordLists[0])->Set(editor.get_keywords(Sql_editor::General_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[3])->Set(editor.get_function_names());
-    ((SCI_WRAPPER_NS WordList*) keywordLists[5])->Set(editor.get_keywords(Sql_editor::Procedure_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[6])->Set(editor.get_keywords(Sql_editor::Client_Keywords));
-    ((SCI_WRAPPER_NS WordList*) keywordLists[7])->Set(editor.get_keywords(Sql_editor::User_Keywords));
+    ((SCI_WRAPPER_NS WordList*) keywordLists[0])->Set(keywords["Major Keywords"].c_str());
+    ((SCI_WRAPPER_NS WordList*) keywordLists[3])->Set(keywords["Functions"].c_str());
+    ((SCI_WRAPPER_NS WordList*) keywordLists[5])->Set(keywords["Procedure keywords"].c_str());
+    ((SCI_WRAPPER_NS WordList*) keywordLists[6])->Set(keywords["User Keywords 1"].c_str());
+
+    // TODO: adjust CSS files so the color values correspond to what the user has set currently.
   }
 
   return result;
@@ -788,7 +795,7 @@ const string markupFromStyle(int style)
 
 //--------------------------------------------------------------------------------------------------
 
-void set_ddl(TemplateDictionary *target, SQLGeneratorInterfaceWrapper* sqlgenModule, 
+void set_ddl(TemplateDictionary *target, SQLGeneratorInterfaceImpl* sqlgenModule, 
              const GrtNamedObjectRef& object, const Scintilla::LexerModule* lexer,
              bool ddl_enabled)
 {
@@ -1016,11 +1023,12 @@ int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::Di
   int total_sp_count= 0;
   int total_trigger_count= 0;
 
-  SQLGeneratorInterfaceWrapper *sqlgenModule= NULL;
+  SQLGeneratorInterfaceImpl *sqlgenModule= NULL;
 
   if (show_ddl)
   {
-    vector<SQLGeneratorInterfaceWrapper*> genmodules= grt->get_implementing_modules<SQLGeneratorInterfaceWrapper>();
+    /*
+    vector<SQLGeneratorInterfaceImpl*> genmodules= grt->get_implementing_modules<SQLGeneratorInterfaceWrapper>();
     for (vector<SQLGeneratorInterfaceWrapper*>::const_iterator iter= genmodules.begin();
          iter != genmodules.end(); ++iter)
     {
@@ -1029,7 +1037,8 @@ int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::Di
         sqlgenModule= *iter;
         break;
       }
-    }
+    }*/
+    sqlgenModule = dynamic_cast<SQLGeneratorInterfaceImpl*>(grt->get_module("DbMySQL"));
     if (!sqlgenModule)
       throw logic_error("could not find SQL generation module for mysql");
   }
@@ -1256,7 +1265,6 @@ int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::Di
             if (bad_list.size() > 0)
             {
               grt->send_error("Syntax errors found in file:");
-              ctemplate::TemplateNamelist::SyntaxListType::iterator iterator= bad_list.begin();
               for (ctemplate::TemplateNamelist::SyntaxListType::iterator iterator= bad_list.begin(); 
                 iterator != bad_list.end(); iterator++)
                   grt->send_error(*iterator);
