@@ -1,3 +1,22 @@
+/*
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
 #include "stdafx.h"
 
 #include "grt/grt_manager.h"
@@ -81,7 +100,7 @@ public:
     _options_box.add(&_generate_create_index_check, false, false);
     _generate_show_warnings_check.set_text(_("Add SHOW WARNINGS After Every DDL Statement"));
     _options_box.add(&_generate_show_warnings_check, false, false);
-    _skip_users_check.set_text(_("Do Not Create Users. Only Create Privileges"));
+    _skip_users_check.set_text(_("Do Not Create Users. Only Create Privileges (GRANTs)"));
     _options_box.add(&_skip_users_check, false, false);
     _generate_insert_check.set_text(_("Generate INSERT Statements for Tables"));
     _options_box.add(&_generate_insert_check, false, false);
@@ -179,6 +198,16 @@ protected:
   {
     if (advancing)
       setup_filters();
+    std::vector<std::string> schemata;
+    try
+    {
+      _export_be->load_schemata(schemata);
+    }
+    catch (const std::exception &exc)
+    {
+      mforms::Utilities::show_error("Connect to Database", "Error connecting to database:\n" + std::string(exc.what()), "OK", "", "");
+      return;
+    }
     
     WizardObjectFilterPage::enter(advancing);
   }
@@ -229,6 +258,7 @@ protected:
   
   virtual bool advance()
   {
+    _export_be->set_db_options( _export_be->load_db_options());
     _export_be->set_option("OutputFileName", values().get_string("OutputFileName"));
     _export_be->set_option("GenerateDrops", values().get_int("GenerateDrops"));
     _export_be->set_option("SkipForeignKeys", values().get_int("SkipForeignKeys"));
@@ -328,11 +358,12 @@ public:
     : ConnectionPage(form, name)
   {
   }
-
+  /*
   virtual std::string next_button_caption()
   {
     return execute_caption();
   }
+  */
 
   void load_saved_connection()
   {
@@ -422,9 +453,8 @@ public:
 
   bool do_connect()
   {
-    static const grt::ValueRef vref;
-    boost::function<grt::ValueRef (bool)> task = boost::lambda::var(vref);
-    execute_grt_task(boost::bind(task,boost::bind(&DbConnection::test_connection, ((WbPluginDbExport*)_form)->be()->db_conn())), false);
+    execute_grt_task(boost::bind(boost::function<grt::ValueRef (bool)> (boost::lambda::constant(grt::ValueRef())),
+        boost::bind(&DbConnection::test_connection, ((WbPluginDbExport*)_form)->be()->db_conn())), false);
     return true;
   }  
   
@@ -467,10 +497,10 @@ WbPluginDbExport::WbPluginDbExport(grt::Module *module)
   _selection_page= new ExportFilterPage(this, &_db_frw_eng);
   _progress_page= new ExportProgressPage(this);
   _progress_page->set_connection_page((MyConnectionPage*)_connection_page);
+  add_page(mforms::manage(_connection_page));
   add_page(mforms::manage(_input_page));
   add_page(mforms::manage(_selection_page));
   add_page(mforms::manage(_preview_page));
-  add_page(mforms::manage(_connection_page));
   add_page(mforms::manage(_progress_page));
 
   set_title(_("Forward Engineer to Database"));
